//*************************************************************
//*** FUNCTIONS FOR EEPROM READ/WRITE ON I2C COMMUNICATION  ***
//*************************************************************
//Controller: ATmega32 (Clock: 8 Mhz-internal)
//Compiler	: AVR-GCC (winAVR with AVRStudio)
//Version 	: 2.4
//Author	: CC Dharmani, Chennai (India)
//			  www.dharmanitech.com
//Date		: 17 May 2010
//********************************************************

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "rtc_routines.h"
#include "i2c_routines.h"

//**************************
//Function to initialize RTC
//**************************

unsigned char rtc_init(){
	
	unsigned char errorStatus;
	
	errorStatus = i2c_start();
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	}
	
	errorStatus = i2c_sendAddress(DS1307_W);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	} 
	
	errorStatus = i2c_sendData(0x00);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	}
	
	
	i2c_stop();
	return(!RTC_ERROR);
	
	
}

//***********************************************************************
//Function to read RTC registers and store them in buffer rtc_register[]
//***********************************************************************    
unsigned char rtc_read(RTC_DATE_TIME *r)
{
	
	unsigned char errorStatus, i, data;
	
	errorStatus = i2c_start();
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	}
	
	errorStatus = i2c_sendAddress(DS1307_W);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	}
	
	errorStatus = i2c_sendData(0x00);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	}
	
	errorStatus = i2c_repeatStart();
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	} 
	
	errorStatus = i2c_sendAddress(DS1307_R);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	} 
	
	for(i=0;i<7;i++)
	{
		if(i == 6)  	 
			data = i2c_receiveData_NACK();
		else
			data = i2c_receiveData_ACK();
		
		if(data == ERROR_CODE)
		{
			i2c_stop();
			return(RTC_ERROR);
		}
		
		r->rtc_register[i] = data;
	}
	
	i2c_stop();
	
	
	r->time[8] = 0x00;
	r->time[7] = (r->rtc_register[0] & 0x0f);
	r->time[6] = ((r->rtc_register[0] & 0x70) >> 4);
	r->time[5] = ':';
	r->time[4] = (r->rtc_register[1] & 0x0f);
	r->time[3] = ((r->rtc_register[1] & 0x70) >> 4);
	r->time[2] = ':'; 
	r->time[1] = (r->rtc_register[2] & 0x0f);
	r->time[0] = ((r->rtc_register[2] & 0x30) >> 4);
	
	r->date[11] = 0x00;
	r->date[10] = 0x00;
	r->date[9] = (r->rtc_register[6] & 0x0f);
	r->date[8] = ((r->rtc_register[6] & 0xf0) >> 4);
	r->date[7] = 0;
	r->date[6] = 2;
	r->date[5] = '.';
	r->date[4] = (r->rtc_register[5] & 0x0f);
	r->date[3] = ((r->rtc_register[5] & 0x10) >> 4);
	r->date[2] = '.';
	r->date[1] = (r->rtc_register[4] & 0x0f);
	r->date[0] = ((r->rtc_register[4] & 0x30) >> 4);
	return (!RTC_ERROR);
}	  

//*******************************
//Function to write RTC registers
//*******************************
unsigned char rtc_write (RTC_DATE_TIME *r)
{
	unsigned char errorStatus, i;
	
	r->rtc_register[0] = ((r->time[6] & 0x07) << 4) | (r->time[7] & 0x0f);
	r->rtc_register[1] = ((r->time[3] & 0x07) << 4) | (r->time[4] & 0x0f);
	r->rtc_register[2] = ((r->time[0] & 0x03) << 4) | (r->time[1] & 0x0f);  
	r->rtc_register[3] = r->date[10];
	r->rtc_register[4] = ((r->date[0] & 0x03) << 4) | (r->date[1] & 0x0f);
	r->rtc_register[5] = ((r->date[3] & 0x01) << 4) | (r->date[4] & 0x0f);
	r->rtc_register[6] = ((r->date[8] & 0x0f) << 4) | (r->date[9] & 0x0f);
	
	
  	errorStatus = i2c_start();
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	} 
	
	errorStatus = i2c_sendAddress(DS1307_W);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	} 
	
	errorStatus = i2c_sendData(0x00);
	if(errorStatus == RTC_ERROR)
	{
		i2c_stop();
		return(RTC_ERROR);
	} 
	
    for(i=0;i<7;i++)
    {
		errorStatus = i2c_sendData(r->rtc_register[i]);  
		if(errorStatus == RTC_ERROR)
		{
			i2c_stop();
			return(RTC_ERROR);
		} 
	}
	
	i2c_stop();
	return(!RTC_ERROR);
}

// ******************************************************************
// Function to get RTC date & time in FAT32 format
// ******************************************************************   
unsigned char getDateTime_FAT(void)
{
  
	unsigned char mth, dt, hr, min, sec, errorStatus; 
	unsigned int yr;
	
	RTC_DATE_TIME rtc;
	
	errorStatus = rtc_read(&rtc);
	if(errorStatus) return 1;

	yr = (rtc.rtc_register[6] & 0xf0) >> 4;
	yr = (yr * 10)+(rtc.rtc_register[6] & 0x0f);
	yr = yr+2000;
	yr = yr - 1980;

	dateFAT = yr;

	mth = (rtc.rtc_register[5] & 0xf0) >> 4;
	mth = (mth * 10)+(rtc.rtc_register[5] & 0x0f);

	dateFAT = (dateFAT << 4) | mth;

	dt = (rtc.rtc_register[4] & 0xf0) >> 4;
	dt = (dt * 10)+(rtc.rtc_register[4] & 0x0f);

	dateFAT = (dateFAT << 5) | dt;


	hr = (rtc.rtc_register[2] & 0xf0) >> 4;
	hr = (hr * 10)+(rtc.rtc_register[2] & 0x0f);

	timeFAT = hr;

	min = (rtc.rtc_register[1] & 0xf0) >> 4;
	min = (min * 10)+(rtc.rtc_register[1] & 0x0f);

	timeFAT = (timeFAT << 6) | min;

	sec = (rtc.rtc_register[0] & 0xf0) >> 4;
	sec = (sec * 10)+(rtc.rtc_register[0] & 0x0f);
	sec = sec / 2;    //FAT32 fromat accepts dates with 2sec resolution (e.g. value 5 => 10sec)

	timeFAT = (timeFAT << 5) | sec;


	return 0;
}
